==> ben_cogs-0.13.4.tar.gz: ben_cogs-0.13.4/ben_cogs/bot.py <==
import asyncio
import contextlib
import logging
import re
import traceback
import uuid

try:
	import asyncpg
except ImportError:
	HAVE_ASYNCPG = False
else:
	HAVE_ASYNCPG = True
import discord
from discord.ext import commands
try:
	import uvloop
except ImportError:
	pass  # Windows
else:
	asyncio.set_event_loop_policy(uvloop.EventLoopPolicy())

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger('bot')

class BenCogsBot(commands.AutoShardedBot):
	def __init__(self, *args, **kwargs):
		self.config = kwargs.pop('config')
		self._should_setup_db = kwargs.pop('setup_db')
		if self._should_setup_db and not HAVE_ASYNCPG:
			raise ImportError('this bot requires asyncpg but it is not installed')
		self.process_config()
		self._fallback_prefix = str(uuid.uuid4())

		super().__init__(
			command_prefix=self.get_prefix_,
			description=self.config.get('description'),
			help_command=commands.MinimalHelpCommand(),
			activity=self.activity,
			*args, **kwargs)

	def process_config(self):
		self.owners = set(self.config.get('owners', []))
		self.config['success_emojis'] = {
			k: convert_emoji(v)
			for k, v
			in self.config.get('success_emojis', {False: '❌', True: '✅'}).items()}

	@property
	def activity(self):
		try:
			prefixes = self.config['prefixes']
		except KeyError:
			return None

		return prefixes and discord.Game(name=prefixes[0] + 'help')

	def get_prefix_(self, bot, message):
		match = self.prefix_re.search(message.content)

		if match is None:
			# Callable prefixes must always return at least one prefix,
			# but no prefix was found in the message,
			# so we still have to return *something*.
			# Use a UUID because it's practically guaranteed not to be in the message.
			return self._fallback_prefix
		else:
			return match[0]

	@property
	def prefix_re(self):
		prefixes = self.config.get('prefixes', [])

		prefixes = list(prefixes)  # ensure it's not a tuple
		if self.is_ready():
			prefixes.extend([f'<@{self.user.id}>', f'<@!{self.user.id}>'])

		prefixes = '|'.join(map(re.escape, prefixes))
		prefixes = f'(?:{prefixes})'

		return re.compile(fr'{prefixes}\s*', re.IGNORECASE)

	### Events

	async def on_ready(self):
		separator = '━' * 44
		logger.info(separator)
		logger.info('Logged in as: %s', self.user)
		logger.info('ID: %s', self.user.id)
		logger.info(separator)

	async def on_message(self, message):
		if self.should_reply(message):
			await self.process_commands(message)

	async def process_commands(self, message):
		# overridden because the default process_commands ignores bots now
		ctx = await self.get_context(message)
		await self.invoke(ctx)

	# based on https://github.com/Rapptz/RoboDanny/blob/ca75fae7de132e55270e53d89bc19dd2958c2ae0/bot.py#L77-L85
	async def on_command_error(self, ctx, error):
		if isinstance(error, commands.NoPrivateMessage):
			await ctx.author.send('This command cannot be used in private messages.')
		elif isinstance(error, commands.DisabledCommand):
			message = 'Sorry. This command is disabled and cannot be used.'
			try:
				await ctx.author.send(message)
			except discord.Forbidden:
				await ctx.send(message)
		elif isinstance(error, commands.NotOwner):
			logger.error('%s tried to run %s but is not the owner', ctx.author, ctx.command.name)
			await ctx.message.add_reaction(self.config['success_emojis'][False])
		elif isinstance(error, (commands.UserInputError, commands.CheckFailure)):
			await ctx.send(error)
		elif (
			isinstance(error, commands.CommandInvokeError)
			and (not ctx.cog or type(ctx.cog).cog_command_error is commands.Cog.cog_command_error)  # not overridden
			and not hasattr(ctx.command, 'on_error')
		):
			logger.error('"%s" caused an exception <%s>', ctx.message.content, ctx.message.jump_url)
			logger.error(''.join(traceback.format_tb(error.original.__traceback__)))
			# pylint: disable=logging-format-interpolation
			logger.error('{0.__class__.__name__}: {0}'.format(error.original))

			await ctx.send('An internal error occured while trying to run that command.')

	### Utility functions

	def should_reply(self, message):
		"""return whether the bot should reply to a given message"""
		return not (
			message.author == self.user
			or (message.author.bot and not self.should_reply_to_bot(message)))

	def should_reply_to_bot(self, message):
		should_reply = not self.config['ignore_bots'].get('default')
		overrides = self.config['ignore_bots']['overrides']

		def check_override(location, overrides_key):
			return location and location.id in overrides[overrides_key]

		if check_override(message.guild, 'guilds') or check_override(message.channel, 'channels'):
			should_reply = not should_reply

		return should_reply

	async def is_owner(self, user):
		return user.id in self.owners or await super().is_owner(user)

	async def is_privileged(self, member):
		return member.guild_permissions.administrator or await self.is_owner(member)

	### Init / Shutdown

	async def start(self):
		if self._should_setup_db:
			await self.init_db()
		self.load_extensions()

		await super().start(self.config['tokens'].pop('discord'))

	async def close(self):
		if self._should_setup_db:
			with contextlib.suppress(AttributeError):
				await self.pool.close()
		await super().close()

	async def init_db(self):
		credentials = self.config['database']
		self.pool = await asyncpg.create_pool(**credentials)

	def load_extensions(self):
		for extension in self.startup_extensions:  # subclasses must define this
			self.load_extension(extension)

def convert_emoji(s) -> discord.PartialEmoji:
	match = re.search(r'<?(a?):([A-Za-z0-9_]+):([0-9]{17,})>?', s)
	if match:
		return discord.PartialEmoji(animated=match[1], name=match[2], id=int(match[3]))
	return discord.PartialEmoji(animated=None, name=s, id=None)

==> ben_cogs-0.13.5.tar.gz: ben_cogs-0.13.5/ben_cogs/bot.py <==
import asyncio
import contextlib
import logging
import re
import traceback
import uuid

try:
	import asyncpg
except ImportError:
	HAVE_ASYNCPG = False
else:
	HAVE_ASYNCPG = True
import discord
from discord.ext import commands
try:
	import uvloop
except ImportError:
	pass  # Windows
else:
	asyncio.set_event_loop_policy(uvloop.EventLoopPolicy())

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger('bot')

class BenCogsBot(commands.AutoShardedBot):
	def __init__(self, *args, **kwargs):
		self.config = kwargs.pop('config')
		self._should_setup_db = kwargs.pop('setup_db', False)
		if self._should_setup_db and not HAVE_ASYNCPG:
			raise ImportError('this bot requires asyncpg but it is not installed')
		self.process_config()
		self._fallback_prefix = str(uuid.uuid4())

		super().__init__(
			command_prefix=self.get_prefix_,
			description=self.config.get('description'),
			help_command=commands.MinimalHelpCommand(),
			activity=self.activity,
			*args, **kwargs)

	def process_config(self):
		self.owners = set(self.config.get('owners', []))
		self.config['success_emojis'] = {
			k: convert_emoji(v)
			for k, v
			in self.config.get('success_emojis', {False: '❌', True: '✅'}).items()}

	@property
	def activity(self):
		try:
			prefixes = self.config['prefixes']
		except KeyError:
			return None

		return prefixes and discord.Game(name=prefixes[0] + 'help')

	def get_prefix_(self, bot, message):
		match = self.prefix_re.search(message.content)

		if match is None:
			# Callable prefixes must always return at least one prefix,
			# but no prefix was found in the message,
			# so we still have to return *something*.
			# Use a UUID because it's practically guaranteed not to be in the message.
			return self._fallback_prefix
		else:
			return match[0]

	@property
	def prefix_re(self):
		prefixes = self.config.get('prefixes', [])

		prefixes = list(prefixes)  # ensure it's not a tuple
		if self.is_ready():
			prefixes.extend([f'<@{self.user.id}>', f'<@!{self.user.id}>'])

		prefixes = '|'.join(map(re.escape, prefixes))
		prefixes = f'(?:{prefixes})'

		return re.compile(fr'{prefixes}\s*', re.IGNORECASE)

	### Events

	async def on_ready(self):
		separator = '━' * 44
		logger.info(separator)
		logger.info('Logged in as: %s', self.user)
		logger.info('ID: %s', self.user.id)
		logger.info(separator)

	async def on_message(self, message):
		if self.should_reply(message):
			await self.process_commands(message)

	async def process_commands(self, message):
		# overridden because the default process_commands ignores bots now
		ctx = await self.get_context(message)
		await self.invoke(ctx)

	# based on https://github.com/Rapptz/RoboDanny/blob/ca75fae7de132e55270e53d89bc19dd2958c2ae0/bot.py#L77-L85
	async def on_command_error(self, ctx, error):
		if isinstance(error, commands.NoPrivateMessage):
			await ctx.author.send('This command cannot be used in private messages.')
		elif isinstance(error, commands.DisabledCommand):
			message = 'Sorry. This command is disabled and cannot be used.'
			try:
				await ctx.author.send(message)
			except discord.Forbidden:
				await ctx.send(message)
		elif isinstance(error, commands.NotOwner):
			logger.error('%s tried to run %s but is not the owner', ctx.author, ctx.command.name)
			await ctx.message.add_reaction(self.config['success_emojis'][False])
		elif isinstance(error, (commands.UserInputError, commands.CheckFailure)):
			await ctx.send(error)
		elif (
			isinstance(error, commands.CommandInvokeError)
			and (not ctx.cog or type(ctx.cog).cog_command_error is commands.Cog.cog_command_error)  # not overridden
			and not hasattr(ctx.command, 'on_error')
		):
			logger.error('"%s" caused an exception <%s>', ctx.message.content, ctx.message.jump_url)
			logger.error(''.join(traceback.format_tb(error.original.__traceback__)))
			# pylint: disable=logging-format-interpolation
			logger.error('{0.__class__.__name__}: {0}'.format(error.original))

			await ctx.send('An internal error occured while trying to run that command.')

	### Utility functions

	def should_reply(self, message):
		"""return whether the bot should reply to a given message"""
		return not (
			message.author == self.user
			or (message.author.bot and not self.should_reply_to_bot(message)))

	def should_reply_to_bot(self, message):
		should_reply = not self.config['ignore_bots'].get('default')
		overrides = self.config['ignore_bots']['overrides']

		def check_override(location, overrides_key):
			return location and location.id in overrides[overrides_key]

		if check_override(message.guild, 'guilds') or check_override(message.channel, 'channels'):
			should_reply = not should_reply

		return should_reply

	async def is_owner(self, user):
		return user.id in self.owners or await super().is_owner(user)

	async def is_privileged(self, member):
		return member.guild_permissions.administrator or await self.is_owner(member)

	### Init / Shutdown

	async def start(self):
		if self._should_setup_db:
			await self.init_db()
		self.load_extensions()

		await super().start(self.config['tokens'].pop('discord'))

	async def close(self):
		if self._should_setup_db:
			with contextlib.suppress(AttributeError):
				await self.pool.close()
		await super().close()

	async def init_db(self):
		credentials = self.config['database']
		self.pool = await asyncpg.create_pool(**credentials)

	def load_extensions(self):
		for extension in self.startup_extensions:  # subclasses must define this
			self.load_extension(extension)

def convert_emoji(s) -> discord.PartialEmoji:
	match = re.search(r'<?(a?):([A-Za-z0-9_]+):([0-9]{17,})>?', s)
	if match:
		return discord.PartialEmoji(animated=match[1], name=match[2], id=int(match[3]))
	return discord.PartialEmoji(animated=None, name=s, id=None)
